%--------------------------------------------------------------------
% Numerical Example of LPV Robust Synthesis Algorithm 
%
% This file is to synthesis a bobust LPV controller for a simple uncertain
% LPV system. The uncertainty system is described by an interconnection of a
% nominal LPV system G_rho and a SISO norm bounded uncertainty Delta.
% Several Weight functions Wu, We and Wd are selected to specify the
% performance and robustness in frequency domain.
%
% Details of the model and the algorithm are in: S. Wang, H. Pfifer and 
% P. Seiler, 'Robust Synthesis for Linear Parameter Varying Systems Using 
% Integral Quadratic Constraints', submitted to IEEE Conference on Decision
% and Control, 2014.
%
% Update:03/18/2014
%--------------------------------------------------------------------
clear all;
clc;
%% Model Construction

rho = [2:0.5:7];  % time varying parameter rho
Np = size(rho,2);

tau = sqrt(133.6-16.8.*rho);  % system parameter tau
C = sqrt(4.8.*rho-8.6);   % system parameter C
 
Ag = -1./tau; % SS model matrices for the plant
Bg = 1./tau;
Cg = C;
Dg = 0;

G = ss;
for i = 1:Np
    Agi = Ag(i);
    Bgi = Bg(i);
    Cgi = Cg(i);
    G(:,:,i) = ss(Agi,Bgi,Cgi,Dg);
end

%% Weighted Interconnection
Wu = makeweight(0.1,10,10); % uncertainty weight
We = makeweight(10,10,0.1); % performance weight
Wd = 0.1;   % uncertainty weight

systemnames = 'G Wu We Wd';
inputvar = '[d; r; u]';
outputvar = '[ Wu; We; r-G]';
input_to_We = '[r-G]';
input_to_Wu = '[u]';
input_to_Wd = '[d]';
input_to_G = '[Wd+u]';
cleanupsysic = 'yes';
Pic = sysic;

% number of measurements and control inputs
nmeas = 1;
ncon = 1;

%% Initial Settings
% initial Psi
Psi11 = 1;
Psi22 = 1;

% iteration variables
ii = 0;           %iteration times
e_tol = 0.01;     % termination condition
gamopt = 0;       % Robust Performance level
gamopt_old = inf; % gamopt in last iteration

%% Perform robust LPV design with gridded approach

while gamopt_old - gamopt > e_tol
    
    % synthesis step: find a nominal controller
    ii=ii+1;
    fprintf('\n Iteration: %i \n', ii);
    
    scaleinput = blkdiag(inv(Psi22),1,1);
    scaleoutput = blkdiag(Psi11,1,1);
    G = scaleoutput*Pic*scaleinput;         % $\tilde(G_{\rho})$
    
    opt = lpvsynOptions;
    opt.BackOffFactor = 1.2;    % set back off factor for LPV synthesis
    [KGrid,GAMGrid,INFOGrid] = lpvsyneng(G,nmeas,ncon); % synthesis function
   
    fprintf('Gamma controller step: %f \t \n', GAMGrid)
    %%    
    %robust performance analysis step
    
    %get analysis interconnection
    Nic = lft(Pic,KGrid);
    
    % Specify IQCs
    psi{1} = ss(eye(2));
    D2 = ss(tf(1,[1 1]));
    psi{2} = blkdiag(D2,D2);
%     D3 = ss(tf(1,[1 10]));
%     psi{3} = blkdiag(D3,D3);
    
    % options for bisection algorithm
    gamlb = 0;          %lower bound
    gamub = GAMGrid;    %upper bound
    tol = 1e-3;         %tolerance for bisection
    
    while gamub-gamlb>tol
        gamtry = (gamlb+gamub)/2;
        
        %run analysis to find the RP level
        [t,info] = simplerobperf(Nic,[1,1],psi, gamtry);
        
        %new bounds for bisection
        if t <= 0
            gamub = gamtry;
        else
            gamlb = gamtry;
        end
    end
    
    if ii==1
        gamopt = gamub;
    else
        gamopt_old = gamopt;
        gamopt = gamub;
    end
    
    [t,info] = simplerobperf(Nic,[1,1],psi, gamopt);    %run analysis
    
    fprintf('Gamma analysis step: %f \t \n', gamopt)
    
    %%
    % create a new scaled plant for synthesis in the next step
    lam = info.lam;
    M = blkdiag(1,-gamopt^2);
    Pi = ss(zeros(2,2));
    for jj = 1:length(lam)
        Pi = Pi + lam(jj)*psi{jj}'*M*psi{jj};
    end
    
    [Psi,M] = jspec(Pi); %J-spectral Factorization
    
    Psi = Psi*blkdiag(1,1/gamopt);
    
    Psi11 = Psi(1,1);
    Psi22 = Psi(2,2);
    
end

